%%****************************************************
%% corrmat: find the nearest correlation matrix
%%          to a given symmetric matrix. 
%%
%%  [blk,A,C,b] = corrmat(H); 
%%
%%  min || X - H ||_F
%%  s.t.  diag(X) = e, X psd.  
%%
%%  min   t 
%%  s.t.  y + x = svec(H), ||y||_2 <= t.    
%%        diag(X) = e, X psd.  
%%
%% SDPT3: version 3.0 
%% Copyright (c) 1997 by
%% K.C. Toh, M.J. Todd, R.H. Tutuncu
%% Last modified: 18 Dec 01
%%****************************************************

  function [blk,A,C,b] = corrmat(H); 

  if size(H,1) ~= size(H,2); error('corrmat: matrix must be square'); end;
  n = length(H); 
  blktmp{1,1} = 's'; blktmp{1,2} = n;
  h = svec(blktmp,H); 
  
  n2 = n*(n+1)/2; 
  blk{1,1}= 'q'; blk{1,2} = n2+1; 
  A{1,1} = [sparse(n2+n,1) speye(n2+n,n2)];

  blk{2,1}= 's'; blk{2,2} = n; 
  A{2,1} = sparse(n2+n,n2); 
  A{2}(1:n2,:) = speye(n2,n2); 
  tmp = zeros(n,3);
  for k = 1:n
     jk = k*(k-1)/2 + k ; 
     tmp(k,:) = [k jk 1];
  end 
  A{2}(n2+[1:n],:) = spconvert(tmp);  

  b = [h; ones(n,1)];
  C{1,1} = eye(n2+1,1);  
  C{2,1} = sparse(n,n);
  A{1} = A{1}';
  A{2} = A{2}';
%%
%%****************************************************
