function r = sdmrank(A,tol)
%SDMRANK - Sparse Matrix rank.
%   sdmrank(A) provides an estimate of the number of linearly
%   independent rows or columns of a sparse matrix A.
%   sdmrank(A,tol) is the number of singular values of sparse matrix A
%   that are larger than tol.
%   sdmrank(A) uses the default tol = max(size(A)) * norm(A) * eps. 

%   This file is part of SeDuMi Interface 1.04 (JUL2002)
%   Last update : 31 July 2002
%   Copyright (C) 2002 Dimitri Peaucelle & Krysten Taitz
%   LAAS-CNRS, Toulouse, France
% 
%   This program is free software; you can redistribute it and/or modify
%   it under the terms of the GNU General Public License as published by
%   the Free Software Foundation; either version 2 of the License, or
%   (at your option) any later version.
% 
%   This program is distributed in the hope that it will be useful,
%   but WITHOUT ANY WARRANTY; without even the implied warranty of
%   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%   GNU General Public License for more details.
% 
%   You should have received a copy of the GNU General Public License
%   along with this program; if not, write to the Free Software
%   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. 

s = svds(A);
if nargin==1
   tol = max(size(A)') * max(s) * eps;
end
r = sum(s > tol);
