function sys=double(X)
%DOUBLE Returns current numerical value 

% Author Johan Lfberg 
% $Id: double.m,v 1.9 2005/01/05 21:47:36 johanl Exp $  

% Definition of nonlinear variables
mt = yalmip('monomtable');
mt_t = mt';
solution = sdpvar('getSolution');
lmi_variables = X.lmi_variables;
x_variables = depends(X);
opt_variables = solution.variables;
nonlinears = setdiff(lmi_variables,x_variables);

% All double values
values = ones(size(mt,1),1)*nan;
values(solution.variables) = solution.optvar;

% Evaluate the extended operators
extended_variables = find(ismember(X.lmi_variables,yalmip('extvariables')));
if ~isempty(extended_variables)
    for i = 1:length(extended_variables)
        extvar = lmi_variables(extended_variables(i));
        extstruct = yalmip('extstruct',extvar);
        
        for k = 1:length(extstruct.arg)
            if isa(extstruct.arg{k},'sdpvar')
                extstruct.arg{k} = double(extstruct.arg{k});
             end
        end
        
        switch extstruct.fcn
            case 'mpower'      % 
                val = extstruct.arg{1};
            case {'max','min'} % Cannot take several inputs, put everything in a vector
                val = feval(extstruct.fcn,[extstruct.arg{:}]);        
            case {'or','and'}
                try
                    val = feval(extstruct.fcn,extstruct.arg{:});        
                catch
                    val = nan;
                end
                
                
            otherwise          % VAL = OPERATOR(X1,X2,...)
                val = feval(extstruct.fcn,extstruct.arg{:});        
        end
        values(extvar) = full(val);
    end
end

if ~isempty(nonlinears)
    use_these = find(ismember(lmi_variables,nonlinears));
    for i=use_these
        monom_i = mt_t(:,lmi_variables(i));
        used_in_monom = find(monom_i);

        extended_variables = find(ismember(used_in_monom,yalmip('extvariables')));
        if ~isempty(extended_variables)
            for ii = 1:length(extended_variables)
                extvar = used_in_monom(extended_variables(ii));
                extstruct = yalmip('extstruct',extvar);
                for k = 1:length(extstruct.arg)
                    if isa(extstruct.arg{k},'sdpvar')
                        extstruct.arg{k} = double(extstruct.arg{k});
                    end
                end

                switch extstruct.fcn
                    case 'mpower'      %
                        val = extstruct.arg{1};
                    case {'max','min'} % Cannot take several inputs, put everything in a vector
                        val = feval(extstruct.fcn,[extstruct.arg{:}]);
                    otherwise          % VAL = OPERATOR(X1,X2,...)
                        val = feval(extstruct.fcn,extstruct.arg{:});
                end
                values(extvar) = val;

            end
        end
        the_product = 1;
        for j = 1:length(used_in_monom)
            the_product = the_product*values(used_in_monom(j))^monom_i(used_in_monom(j));
        end
        values(lmi_variables(i)) = the_product;
    end
end
sys = X.basis*[1;values(lmi_variables(:))];

if X.typeflag==10
    sys = eig(full(reshape(sys,X.n,X.m)));
else
    sys = full(reshape(sys,X.n,X.m));
end
