function YESNO = is(X,property,additional)
%IS Check property of variable.
%   d = IS(x,property) returns 1 if 'property' holds
%
%   Properties possible to test are: 'real', 'symmetric', 'hermitian',
%   'scalar', 'linear', 'homogeneous', 'integer', 'binary'

% Author Johan Lfberg 
% $Id: is.m,v 1.8 2005/01/05 21:47:36 johanl Exp $   

switch property
    case 'logic'
        YESNO = X.typeflag==12;
    case 'binary'
        YESNO = any(ismember(depends(X),yalmip('binvariables')));
    case 'integer'
        YESNO = any(ismember(depends(X),yalmip('intvariables')));
    case 'real'
        YESNO = isreal(X);
    case 'complex'
        YESNO = ~isreal(X.basis);
    case 'symmetric'
        YESNO = issymmetric(X);
    case 'hermitian'
        YESNO = ishermitian(X);
    case 'scalar'
        YESNO = (X.n*X.m)==1;
    case 'compound'
        YESNO = any(ismember(getvariables(X),yalmip('extvariables')));       
    case 'linear'
        [monomtable,variabletype] = yalmip('monomtable');
        YESNO = ~any(variabletype(getvariables(X)));
        
         %monomtable = monomtable(getvariables(X),:);
         %YESNO = ~(any(find(any(0>monomtable,2) | any(monomtable-fix(monomtable),2))));   
         %YESNO = YESNO &  all(sum(monomtable,2)<=1);
         
         %if ~isequal(YESNO,YESNO1)
         %    error
         %end
         
       % YESNO = islinear(X);   
    case 'bilinear'
         monomtable = yalmip('monomtable');
         monomtable = monomtable(getvariables(X),:);
         YESNO = ~(any(find(any(0>monomtable,2) | any(monomtable-fix(monomtable),2))));   
         YESNO = YESNO & all(sum(monomtable,2)<=2);
         YESNO = YESNO & all(max(monomtable,[],2)<=1);
     case 'quadratic'
         YESNO = (degree(X)==2);   
    case 'lpcone'
            base = X.basis;
          YESNO = all(base(:,1)==0); % No constant
          base = base(:,2:end);
          YESNO = YESNO & all(sum(base,2)==1); 
          YESNO = YESNO & all(sum(base,1)==1);
          YESNO = YESNO & all(sum(base~=0,2)==1);
          YESNO = YESNO & all(sum(base~=0,1)==1);
          YESNO = full(YESNO);

    case 'sdpcone'
        base = X.basis;
        n = X.n;
        YESNO = full(issymmetric(X) & nnz(base)==n*n & all(sum(base,2)==1) & all(base(:,1)==0)) & length(X.lmi_variables)==n*(n+1)/2 & isreal(base); 
    case 'sigmonial'
          monomtable = yalmip('monomtable');
          monomtable = monomtable(getvariables(X),:);
          YESNO = any(find(any(0>monomtable,2) | any(monomtable-fix(monomtable),2)));   
    case 'nonlinear'
        YESNO = ~islinear(X);   
    case 'homogeneous'    
        [sqrList,CompressedList] = yalmip('nonlinearvariables');
        [LinearTerms,NonlinearVariables] = getvariables(X,'both');
        if isempty(NonlinearVariables)
            YESNO = nnz(getbasematrix(X,0))==0;
        else
            % No linear terms
            YESNO = isempty(LinearTerms);
            % No constant terms
            YESNO = YESNO & (nnz(getbasematrix(X,0))==0);
            % Largest degree+1
            maxdegree = sum(any(CompressedList(find(ismember(CompressedList,NonlinearVariables)),:),1));
            % All same degree
            YESNO = YESNO & all(all(CompressedList(find(ismember(CompressedList,NonlinearVariables)),2:maxdegree)>0));
        end
    case 'sos'
        YESNO = (X.typeflag==11);        
    otherwise
        error('Wrong input argument.');
end

YESNO = full(YESNO);


