function Z = mtimes(X,Y)
%MTIMES (overloaded)

% Author Johan Lfberg
% $Id: mtimes.m,v 1.16 2005/01/11 12:36:47 johanl Exp $

% Check dimensions
X_is_spdvar = isa(X,'sdpvar');
Y_is_spdvar = isa(Y,'sdpvar');

if isempty(X)
    YY = full(reshape(Y.basis(:,1),Y.n,Y.m));
    Z = X*YY;
    return
elseif isempty(Y)
    XX = full(reshape(X.basis(:,1),X.n,X.m));
    Z = XX*Y;
    return
end

switch 2*X_is_spdvar+Y_is_spdvar
    case 3
        if (X.typeflag==5) & (Y.typeflag==5)
            error('Product of norms not allowed');
        end

        try
            x_isscalar =  (X.n*X.m==1);
            y_isscalar =  (Y.n*Y.m==1);

            all_lmi_variables = uniquestripped([X.lmi_variables Y.lmi_variables]);
            Z = X;Z.n = 1;Z.m = 1;Z.lmi_variables = all_lmi_variables;Z.basis = [];

            % Awkward code due to bug in ML6.5
            Xbase = reshape(X.basis(:,1),X.n,X.m);
            Ybase = reshape(Y.basis(:,1),Y.n,Y.m);
            if x_isscalar
                Xbase = sparse(full(Xbase));
            end
            if y_isscalar
                Ybase = sparse(full(Ybase));
            end
           
            % This is what we want, but not supported in 5.3.1
            %index_X = ismembc2(all_lmi_variables,X.lmi_variables);
            %index_Y = ismembc2(all_lmi_variables,Y.lmi_variables);
            
            index_Y = zeros(length(all_lmi_variables),1);
            index_X = zeros(length(all_lmi_variables),1);

            for j = 1:length(all_lmi_variables)
                indexy = find(all_lmi_variables(j)==Y.lmi_variables);
                indexx = find(all_lmi_variables(j)==X.lmi_variables);
                if ~isempty(indexy)
                    index_Y(j) = indexy;
                end
                if ~isempty(indexx)
                    index_X(j) = indexx;
                end
            end

            ny = Y.n;
            my = Y.m;
            nx = X.n;
            mx = X.m;

            Z.lmi_variables = [Z.lmi_variables zeros(1,length(X.lmi_variables)*length(Y.lmi_variables))];

            % Linear terms
            base = Xbase*Ybase;
            Z.basis = base(:);
            x_base_not_zero = nnz(Xbase)>0;
            y_base_not_zero = nnz(Ybase)>0;
            for i = 1:length(all_lmi_variables)
                base = 0;
                if index_Y(i) & x_base_not_zero
                    base = Xbase*getbasematrixwithoutcheck(Y,index_Y(i));
                end
                if index_X(i) & y_base_not_zero
                    base = base + getbasematrixwithoutcheck(X,index_X(i))*Ybase;
                end
                Z.basis(:,i+1) = base(:);
            end

            % Nonlinear terms
            i = i+1;

            mt = yalmip('monomtable');
            oldmt = mt;
            nvar = length(all_lmi_variables);
            local_mt = mt(all_lmi_variables,:);

            used_variables = find((any(local_mt,1)));            
            local_mt = local_mt(:,used_variables);
            theyvars = find(index_Y);
            thexvars = find(index_X);

            hash = randn(size(mt,2),1);
            mt_hash = mt*hash;
            hash = hash(used_variables);

            for ix = thexvars(:)'

                if mx==1
                    Xibase = X.basis(:,1+index_X(ix));
                else
                    Xibase = reshape(X.basis(:,1+index_X(ix)),nx,mx);
                end                
                mt_x = local_mt(ix,:);

                if my==-1
                    Yibase = Xibase*Y.basis; %Matrix vector, complete basis in one shot
                end

                for iy = theyvars(:)'
                    if my==-1
                        prodbase = Yibase(:,1+index_Y(iy));
                    else
                        ff=Y.basis(:,1+index_Y(iy));
                        Yibase = reshape(ff,ny,my);
                        prodbase = Xibase*Yibase;
                    end
                    if (norm(prodbase,inf)>1e-12)

                        mt_y = local_mt(iy,:);
                        % Idiot-hash the lists
                        mt_xplusy = mt_x+mt_y;
                        new_hash = mt_xplusy*hash;

                        if abs(new_hash)<eps
                            Z.basis(:,1) = Z.basis(:,1) + prodbase(:);
                        else

                            before = find(mt_hash==new_hash);

                            if isempty(before)
                                mt(end+1,used_variables) = mt_xplusy;
                                mt_hash = [mt_hash;new_hash];                                
                                Z.lmi_variables(i) = size(mt,1);
                            else                                
                                Z.lmi_variables(i) = before;
                            end
                            Z.basis(:,i+1)  = prodbase(:);i = i+1;
                        end
                    end
                end
            end

            % We pre-allocated a sufficiently long, now picke the ones we
            % actually filled with values
            Z.lmi_variables = Z.lmi_variables(1:i-1);

            % Fucked up order (lmi_variables should be sorted)
            if any(diff(Z.lmi_variables)<0)
                [i,j]=sort(Z.lmi_variables);
                Z.lmi_variables = Z.lmi_variables(j);
                Z.basis(:,2:end) = Z.basis(:,j+1);
            end

            %[un_Z_vars] = uniquestripped(Z.lmi_variables);
            [un_Z_vars,hh,jj] = unique(Z.lmi_variables);
            if length(Z.lmi_variables) ~=length(un_Z_vars)                
                Z.basis = Z.basis*sparse([1 1+jj],[1 1+(1:length(jj))],ones(1,1+length(jj)))';                               
                Z.lmi_variables = un_Z_vars;
            end

            if ~isequal(mt,oldmt)
                yalmip('setmonomtable',mt);
            end

            if ~(x_isscalar | y_isscalar)
                Z.n = X.n;
                Z.m = Y.m;
            else
                Z.n = max(X.n,Y.n);
                Z.m = max(X.m,Y.m);
            end
        catch
            error(lasterr)
        end


    case 2

        n_X = X.n;
        m_X = X.m;
        [n_Y,m_Y] = size(Y);

        x_isscalar =  (n_X*m_X==1);
        y_isscalar =  (n_Y*m_Y==1);

        if ~x_isscalar
            if ((m_X~= n_Y & ~y_isscalar))
                error('Inner matrix dimensions must agree.')
            end
        end

        if ((~y_isscalar) & (X.typeflag == 5)) | ((y_isscalar) & (X.typeflag == 5) & (Y<0))
            error('Norm expression can only be multiplied with non-negative scalar');
        end

        n = n_X;
        m = m_Y;
        Z = X;

        if x_isscalar
            Z.n = n_Y;
            Z.m = m_Y;
            Z.basis = kron(Z.basis,Y(:));
            Z = clean(Z);
            return
        elseif y_isscalar
            Z.n = n_X;
            Z.m = m_X;
            Z.basis = Z.basis*Y;
            Z = clean(Z);
            return
        end

        Z.n = n;
        Z.m = m;
        Z.basis = kron(Y.',speye(n_X))*X.basis;

    case 1

        n_Y = Y.n;
        m_Y = Y.m;
        [n_X,m_X] = size(X);

        x_isscalar =  (n_X*m_X==1);
        y_isscalar =  (n_Y*m_Y==1);

        if ~x_isscalar
            if ((m_X~= n_Y & ~y_isscalar))
                error('Inner matrix dimensions must agree.')
            end
        end

        if ((~x_isscalar) & (Y.typeflag == 5)) | ((x_isscalar) & (Y.typeflag == 5) & (X<0))
            error('Norm expression can only be multiplied with non-negative scalar');
        end

        n = n_X;
        m = m_Y;
        Z = Y;


        % Special cases
        if x_isscalar
            Z.n = n_Y;
            Z.m = m_Y;               
            try
                Z.basis = sparse(X)*Y.basis;
            catch
                % This works better when low on memory in some cases
                [i,j,k] = find(Y.basis);                
                Z.basis = sparse(i,j,X*k,size(Y.basis,1),size(Y.basis,2));                
            end
            
            Z = clean(Z);
            return
        elseif y_isscalar
            Z.n = n_X;
            Z.m = m_X;
            Z.basis = X(:)*Y.basis;
            Z = clean(Z);
            return
        end

        Z.basis = kron(speye(m_Y),X)*Y.basis;
        Z.n = n;
        Z.m = m;

    otherwise
        error('Logical error in mtimes. Report bug')
end

Z = clean(Z);