function Z = replace(X,Y,W)
%REPLACE Substitutes variables
%
%Z = REPLACE(Y,X,W)  Replaces any occurence of the SDPVAR object Y 
%                    in the SDPVAR object X with the double W
%
% Example
%  x = sdpvar(1,1);
%  t = sdpvar(1,1);
%  Y = [1+t;1+x+t];
%  Y = replace(Y,x,2) generates Y=[1+t;3+t]

% Author Johan Lfberg 
% $Id: replace.m,v 1.3 2004/10/14 12:58:06 johanl Exp $  

if ~isa(X,'sdpvar')
  error('First arguments must be an sdpvar object') 
end
if ~isa(Y,'sdpvar')
  error('Second arguments must be an sdpvar object') 
end
if ~is(Y,'linear')
  error('Second arguments must be linear') 
end
if ~isa(W,'double')
  error('Third arguments must be a double') 
end
if ~isequal(size(Y),size(W))
  error('Both arguments must have same size') 
end

y_lmi_variables = Y.lmi_variables;
b = W(:)-Y.basis(:,1);
A = Y.basis(:,2:end);
feas_var = A\b;
if norm(A*feas_var-b)>sqrt(eps)
  error('Inconsistent assignment')
end

x_lmi_variables = X.lmi_variables;
n = X.n;
m = X.m;

if is(X,'linear')
    Z = X.basis(:,1);    
    for i = 1:length(x_lmi_variables)
        j = find(x_lmi_variables(i) == y_lmi_variables);
        if isempty(j)
            Z = Z + recover(x_lmi_variables(i))*X.basis(:,i+1);
        else
            Z = Z + feas_var(j)*X.basis(:,i+1);
        end
    end
else
    Z = X.basis(:,1);
    x_vars = depends(X);    
    p_exp = exponents(X,recover(x_vars));
    if nnz(p_exp(1,:))==0
        p_exp = p_exp(2:end,:);
    end
    
    % FIX : Vectorize for speed
    for i = 1:size(p_exp,1)
        used_variables = x_vars(find(p_exp(i,:)));
        if isempty(intersect(used_variables,y_lmi_variables))
            Z = Z + recover(x_lmi_variables(i))*X.basis(:,i+1);
        else
            x_i=1; 
            %dummy1 = [];
            %dummy2 = [];
            %dummy3 = 1;
            for j = 1:length(used_variables)
                 y_index = find(used_variables(j)==y_lmi_variables);
                 if isempty(y_index)
                     ui = find(x_vars == used_variables(j));
                     %dummy1 = [dummy1 used_variables(j)];
                     %dummy2 = [dummy2 p_exp(i,ui)];
                     x_i = x_i * recover(used_variables(j))^p_exp(i,ui);
                 else
                     ui = find(x_vars == used_variables(j));
                     %dummy3 = dummy3*feas_var(y_index)^p_exp(i,ui);
                     x_i = x_i* feas_var(y_index)^p_exp(i,ui);
                 end
            end
           % if isempty(dummy1)
           %     x_i = dummy3;
           % else
           % x_i = dummy3*recovermonoms(dummy2,recover(dummy1));
           % end
            Z = Z + x_i*X.basis(:,i+1);            
        end
    end
    
end
if isa(Z,'sdpvar')
Z.n =n;
Z.m = m;
Z.typeflag = X.typeflag;
else
    Z = reshape(full(Z),n,m);
end
    


