function x_opt = plot(varargin)
%plot               plots feasible set
%
% p = plot(F,x,c,n,options) 
%
% F:  set object 
% x:  projected variables [2x1 sdpvar]
% c:  color [char or double]
% n:  #vertices [double ]
% options: options structure from sdpsettings 
 

% Author Johan Lfberg
% $Id: plot.m,v 1.3 2004/09/13 07:00:33 johanl Exp $

% Get the onstraints

if nargin<1
    return
end

F = varargin{1};

if length(F)==0
   return;
end
    
if nargin < 5
    opts = sdpsettings('verbose',0);
else
    opts = varargin{5};
    if isempty(opts)
        opts = sdpsettings('verbose',0);
    end
end

if nargin < 4
    n = 40;
else
    n = varargin{4};
    if isempty(n)
        n = 40;
    end
end

if nargin < 3
    color = 'b';
else
    color = varargin{3};
    if isempty(color)
        color = 'b';
    end
end

if nargin < 2
    x_vars = depends(F);
    x = recover(x_vars);
    x_set = x(1:2);
else
    x_set = varargin{2};
    x_set = x_set(:);
    if isempty(x_set)
        x_vars = depends(F);
        x = recover(x_vars);
        x_set = x(1:2);
    end        
end

if isempty(F)
    return
end

sol = solvesdp(F,sum(x_set),opts);
if sol.problem == 1    
    if nargout==0
        plot(0)
    end
    x_opt =[];
else
    x_opt = [];
    phi = [];
    h = waitbar(0,'Please wait...');
    for i = 0:n-1
        c = [cos(i*2*pi/n);sin(i*2*pi/n)];
        sol = solvesdp(F,c'*x_set,opts);        
        x_opt = [x_opt relaxdouble(x_set)];
        waitbar(i/n,h)
    end
    close(h);
    if nargout == 0
        patch(x_opt(1,:),x_opt(2,:),color);
    end
end










