function new_x = monolist(x,d)
% MONOLIST Generate monomials
%
% y = MONOLIST(x,d,f)
%
% Returns the monomials [1 x(1) x(1)^2 ... x(1)^d(1) x(2) x(1)x(2)  etc...]
%
% do 
%
% >>sdpvar x y z
% >>sdisplay(monolist([x y z],4)) 
%
% to see the logic!
%
%  Input
%     x      : Vector with SDPVAR variables
%     d      : Integers > 0, can be []
%
%  See also SDPVAR, DEGREE

% Author Johan Lfberg
% $Id: monolist.m,v 1.14 2004/10/26 09:02:55 johanl Exp $


% FIX : Optimal generation

% Flatten
x_orig = x(:);
x = x(:)';

if length(d)==1 | all(d(1)==d)
    d = d(1);
    % Powers in monomials
    powers = monpowers(length(x),d);
    total_power = sum(powers,2);
    [i,j] = sort(total_power);
    powers =  powers(j,:);

    % Use fast method for x^alpha
    if isequal(getbase(x),[zeros(length(x),1) eye(length(x))])
       new_x = recovermonoms(powers,x);
       return
    end
    
    % Na, we have things like (c'x)^alpha
    % precalc x^p
    for i = 1:length(x)
        temp = x(i);
        precalc{i,1} = temp;
        for j = 2:1:d
            temp = temp*x(i);
            precalc{i,j} = temp;            
        end
    end

    new_x = [];
    for i = 1:size(powers,1) % All monomials
        temp = 1;
        for j = 1:size(powers,2) % All variables
            if powers(i,j)>0              
                temp = temp*precalc{j,powers(i,j)};                                    
            end
        end

        new_x = [new_x temp];

    end

else

    d = d*ones(1,length(x_orig));

    x = [1 x];

    % Lame loop to generate all combinations
    new_x = 1;
    for j = 1:1:max(d)
        temp = [];
        for i = 1:length(x)
            temp = [temp x(i)*new_x];
        end
        new_x = temp;
        new_x = fliplr(unique(new_x));
        new_degrees = degree(new_x,x(2:end));
        remv = [];
        for i = 1:length(d);
            if new_degrees(i)>=d(i);
                x = recover(setdiff(getvariables(x),getvariables(x_orig(i))));
                x = [1;x(:)];
                remv = [remv i];
            end
        end
        d = d(setdiff(1:length(d),remv));
    end
end
new_x = new_x(:);