function output = calllinprog(interfacedata)

% Author Johan Lfberg 
% $Id: callbintprog.m,v 1.1 2004/06/17 08:40:24 johanl Exp $

% Standard input interface
options = interfacedata.options;
F_struc = interfacedata.F_struc;
c       = interfacedata.c;
K       = interfacedata.K;
Q       = interfacedata.Q;
lb      = interfacedata.lb;
ub      = interfacedata.ub;
x0      = interfacedata.x0;
binary_variables = interfacedata.binary_variables;


showprogress('Calling BINTPROG',options.showprogress);

if isempty(F_struc)
    Aeq = [];
    beq = [];
    A = [];
    b = [];
else
    Aeq = -F_struc(1:1:K.f,2:end);
    beq = F_struc(1:1:K.f,1);        
    A =-F_struc(K.f+1:end,2:end);
    b = F_struc(K.f+1:end,1);   
end
solvertime = clock; 

switch options.verbose
case 0
    options.bintprog.Display = 'off';
case 1
    options.bintprog.Display = 'final';
otherwise
    options.bintprog.Display = 'iter';
end

if options.savedebug
    save bintprogdebug full(c) full(A) full(b) full(Aeq) full(beq) options.quadprog
end

[x,fmin,flag,output] = bintprog(full(c), full(A), full(b), full(Aeq), full(beq), x0,options.linprog);
solvertime = etime(clock,solvertime);
problem = 0;

% No duals
D_struc = [];

% Check, currently not exhaustive...
switch flag
case 1
    problem = 0;
case {0,-4,-5,-6}
    problem = 3;
case -2
    problem = 1;
otherwise
    problem = -1;
end

infostr = yalmiperror(problem,'BINTPROG');       

% Save all data sent to solver?
if options.savesolverinput
    solverinput.A = A;
    solverinput.b = b;
    solverinput.Aeq = Aq;
    solverinput.beq = beq;
    solverinput.c = c;
    solverinput.options = options.linprog;
else
    solverinput = [];
end

% Save all data from the solver?
if options.savesolveroutput
    solveroutput.x = x;
    solveroutput.fmin = fmin;
    solveroutput.flag = flag;
    solveroutput.output=output;
else
    solveroutput = [];
end

% Standard interface 
output.x           = x;
output.problem     = problem;
output.infostr     = infostr;
output.solverinput = solverinput;
output.solveroutput= solveroutput;
output.solvertime  = solvertime;
output.D_struc = D_struc;