function output = callquadprog(interfacedata)

% Author Johan Lfberg 
% $Id: callquadprog.m,v 1.3 2004/10/05 17:40:43 johanl Exp $

% Retrieve needed data
options = interfacedata.options;
F_struc = interfacedata.F_struc;
c       = interfacedata.c;
K       = interfacedata.K;
x0      = interfacedata.x0;
Q       = interfacedata.Q;
lb      = interfacedata.lb;
ub      = interfacedata.ub;

switch options.verbose
case 0
    options.quadprog.Display = 'off';
case 1
    options.quadprog.Display = 'final';
otherwise
    options.quadprog.Display = 'iter';
end
    
showprogress('Calling QUADPROG',options.showprogress);

if ~isempty(F_struc)
    Aeq = -F_struc(1:1:K.f,2:end);
    beq = F_struc(1:1:K.f,1);        
    A =-F_struc(K.f+1:end,2:end);
    b = F_struc(K.f+1:end,1);   
else
    A = [];
    b = [];
    Aeq = [];
    beq = [];
end

if options.savedebug
    ops = options.quadprog;
    save quadprogdebug Q c A b Aeq beq lb ub x0 ops
end

solvertime = clock; 
[x,fmin,flag,output,lambda] = quadprog(2*full(Q), full(c), full(A), full(b), full(Aeq), full(beq), lb, ub, x0,options.quadprog);
if flag==5
    [x,fmin,flag,output,lambda] = quadprog(2*full(Q), full(c), full(A), full(b), full(Aeq), full(beq), lb, ub, [],options.quadprog);
end    

solvertime = etime(clock,solvertime);
problem = 0;

% Internal format for duals
D_struc = [lambda.eqlin;lambda.ineqlin];

% Check, currently not exhaustive...
if flag==0
    problem = 3;
else
    if flag>0
        problem = 0;
    else
        if isempty(x)
            x = repmat(nan,length(c),1);
        end
        if any((A*x-b)>sqrt(eps)) | any( abs(Aeq*x-beq)>sqrt(eps))
            problem = 1; % Likely to be infeasible
        else
            if c'*x<-1e10 % Likely unbounded
                problem = 2;
            else          % Probably convergence issues
                problem = 5;
            end
        end
    end
end
infostr = yalmiperror(problem,'QUADPROG');       

% Save all data sent to solver?
if options.savesolverinput
    solverinput.A = A;
    solverinput.b = b;
    solverinput.Aeq = Aq;
    solverinput.beq = beq;
    solverinput.c = c;
    solverinput.H = Q;
    solverinput.options = options.quadprog;
else
    solverinput = [];
end

% Save all data from the solver?
if options.savesolveroutput
    solveroutput.x = x;
    solveroutput.fmin = fmin;
    solveroutput.flag = flag;
    solveroutput.output=output;
    solveroutput.lambda=lambda;  
else
    solveroutput = [];
end



% Standard interface 
output.x           = x;
output.problem     = problem;
output.infostr     = infostr;
output.solverinput = solverinput;
output.solveroutput= solveroutput;
output.solvertime  = solvertime;
output.D_struc = D_struc;