function solver = solverdefinitions

% ****************************
% Create a default solver
% ****************************
emptysolver.tag     = '';
emptysolver.version = '';
emptysolver.checkfor= {''};
emptysolver.call    = '';
emptysolver.subcall = '';
emptysolver.show    = 1;

emptysolver.objective.linear = 0;
emptysolver.objective.quadratic.convex = 0;
emptysolver.objective.quadratic.nonconvex = 0;
emptysolver.objective.polynomial = 0;
emptysolver.objective.maxdet = 0;
emptysolver.objective.sigmonial = 0;

emptysolver.constraint.equalities.linear     = 0;
emptysolver.constraint.equalities.quadratic  = 0;
emptysolver.constraint.equalities.polynomial = 0;
emptysolver.constraint.equalities.sigmonial  = 0;

emptysolver.constraint.inequalities.elementwise.linear = 0;
emptysolver.constraint.inequalities.elementwise.quadratic.convex = 0;
emptysolver.constraint.inequalities.elementwise.quadratic.nonconvex = 0;
emptysolver.constraint.inequalities.elementwise.polynomial = 0;
emptysolver.constraint.inequalities.elementwise.sigmonial = 0;

emptysolver.constraint.inequalities.semidefinite.linear = 0;
emptysolver.constraint.inequalities.semidefinite.quadratic = 0;
emptysolver.constraint.inequalities.semidefinite.polynomial = 0;
emptysolver.constraint.inequalities.semidefinite.sigmonial = 0;

emptysolver.constraint.inequalities.secondordercone = 0;
emptysolver.constraint.inequalities.rotatedsecondordercone = 0;

emptysolver.constraint.integer = 0;
emptysolver.constraint.binary = 0;

emptysolver.dual = 0;
emptysolver.complex = 0;
emptysolver.parametric = 0;

% **************************************
% Some standard solvers to simplify code
% **************************************

% LP solver
lpsolver = emptysolver;
lpsolver.objective.linear = 1;
lpsolver.constraint.equalities.linear = 1;
lpsolver.constraint.inequalities.elementwise.linear = 1;
lpsolver.dual = 1;

% QP solver
qpsolver = emptysolver;
qpsolver.objective.linear = 1;
qpsolver.objective.quadratic.convex = 1;
qpsolver.constraint.equalities.linear = 1;
qpsolver.constraint.inequalities.elementwise.linear = 1;
qpsolver.dual = 1;

% SDP solver
sdpsolver = emptysolver;
sdpsolver.objective.linear = 1;
sdpsolver.constraint.equalities.linear = 1;
sdpsolver.constraint.inequalities.elementwise.linear = 1;
sdpsolver.constraint.inequalities.semidefinite.linear = 1;
sdpsolver.dual = 1;

% ****************************
% INITIALIZE COUNTER
% ****************************
i = 1;

% ****************************
% DEFINE SOLVERS
% ****************************

solver(i) = lpsolver;
solver(i).tag     = 'glpk';
solver(i).checkfor= {'glpkmex.m'};
solver(i).call    = 'callglpk';
solver(i).constraint.integer = 1;
i = i+1;

solver(i) = lpsolver;
solver(i).tag     = 'cdd';
solver(i).checkfor= {'cddmex'};
solver(i).call    = 'callcdd';
i = i+1;

solver(i) = lpsolver;
solver(i).tag     = 'nag';
solver(i).version = 'e04mbf';
solver(i).checkfor= {'e04mbf'};
solver(i).call    = 'callnage04mbf';
i = i+1;

solver(i) = qpsolver;
solver(i).tag     = 'nag';
solver(i).version = 'e04naf';
solver(i).checkfor= {'e04naf'};
solver(i).call    = 'callnage04naf';
i = i+1;

solver(i) = qpsolver;
solver(i).tag     = 'xpress';
solver(i).checkfor= {'mexpress.m'};
solver(i).call    = 'callmexpress';
solver(i).constraint.integer = 1;
solver(i).constraint.binary = 1;
i = i+1;

solver(i) = lpsolver;
solver(i).tag     = 'qsopt';
solver(i).checkfor= {'qsopt.m'};
solver(i).call    = 'callqsopt';
i = i+1;

solver(i) = qpsolver;
solver(i).tag     = 'clp';
solver(i).checkfor= {'clp.m'};
solver(i).call    = 'callclp';
i = i+1;

solver(i) = qpsolver;
solver(i).tag     = 'mosek';
solver(i).version = 'LP/QP';
solver(i).checkfor= {'mosekopt'};
solver(i).call    = 'callmosek';
i = i+1;

solver(i) = lpsolver;
solver(i).tag     = 'mosek';
solver(i).version = 'SOCP';
solver(i).checkfor= {'mosekopt'};
solver(i).call    = 'callmosek';
solver(i).constraint.inequalities.secondordercone = 1;
i = i+1;

solver(i) = lpsolver;
solver(i).tag     = 'mosek';
solver(i).version = 'GEOMETRIC';
solver(i).checkfor= {'mosekopt'};
solver(i).call    = 'callmosek';
solver(i).objective.sigmonial = 1;
solver(i).constraint.inequalities.elementwise.sigmonial = 1;
solver(i).constraint.equalities.elementwise.nonlinear = 1;
i = i+1;

solver(i) = qpsolver;
solver(i).tag     = 'cplex';
solver(i).version = 'CPLEXINT';
solver(i).checkfor= {'cplexint'};
solver(i).call    = 'callcplexint';
solver(i).constraint.inequalities.elementwise.quadratic.convex = 1;
solver(i).constraint.integer = 1;
solver(i).constraint.binary = 1;
i = i+1;

solver(i) = qpsolver;
solver(i).tag     = 'cplex';
solver(i).version = 'CPLEXMEX';
solver(i).checkfor= {'cplexmex'};
solver(i).call    = 'callcplexmex';
solver(i).constraint.inequalities.elementwise.quadratic.convex = 1;
solver(i).constraint.integer = 1;
solver(i).constraint.binary = 1;
i = i+1;

solver(i) = lpsolver;
solver(i).tag     = 'cplex';
solver(i).version = 'LP-CPLEXINT';
solver(i).checkfor= {'lp_cplex.m'};
solver(i).call    = 'calllp_cplex';
i = i+1;

solver(i) = qpsolver;
solver(i).tag     = 'cplex';
solver(i).version = 'QP-CPLEXINT';
solver(i).checkfor= {'qp_cplex.m'};
solver(i).call    = 'callqp_cplex';
i = i+1;

solver(i) = lpsolver;
solver(i).tag     = 'cplex';
solver(i).version = 'MILP-CPLEXINT';
solver(i).checkfor= {'milp_cplex.m'};
solver(i).call    = 'callmilp_cplex';
solver(i).constraint.integer = 1;
solver(i).constraint.binary = 1;
i = i+1;

solver(i) = qpsolver;
solver(i).tag     = 'cplex';
solver(i).version = 'MIQP-CPLEXINT';
solver(i).checkfor= {'miqp_cplex.m'};
solver(i).call    = 'callmiqp_cplex';
solver(i).constraint.integer = 1;
solver(i).constraint.binary = 1;
i = i+1;

solver(i) = lpsolver;
solver(i).tag     = 'linprog';
solver(i).version = '';
solver(i).checkfor= {'linprog'};
solver(i).call    = 'calllinprog';
i = i+1;

solver(i) = lpsolver;
solver(i).tag     = 'bintprog';
solver(i).version = '';
solver(i).checkfor= {'bintprog.m'};
solver(i).call    = 'callbintprog';
solver(i).constraint.binary = 1;
solver(i).dual = 0;
i = i+1;

solver(i) = qpsolver;
solver(i).tag     = 'quadprog';
solver(i).version = '';
solver(i).checkfor= {'quadprog'};
solver(i).call    = 'callquadprog';
i = i+1;

solver(i) = qpsolver;
solver(i).tag     = 'ooqp';
solver(i).version = '';
solver(i).checkfor= {'ooqp.m'};
solver(i).call    = 'callooqp';
i = i+1;

solver(i) = sdpsolver;
solver(i).tag     = 'sedumi';
solver(i).version = '1.05';
solver(i).checkfor= {'sedumi.m','ada_pcg.m'};
solver(i).call    = 'callsedumi';
solver(i).constraint.equalities.linear = 1;
solver(i).constraint.inequalities.secondordercone = 1;
solver(i).constraint.inequalities.rotatedsecondordercone = 1;
solver(i).complex = 1;
i = i+1;

solver(i) = sdpsolver;
solver(i).tag     = 'sedumi';
solver(i).version = '1.03';
solver(i).checkfor= {'sedumi.m','doinfac.m'};
solver(i).call    = 'callsedumi';
solver(i).constraint.equalities.linear = 1;
solver(i).constraint.inequalities.secondordercone = 1;
solver(i).constraint.inequalities.rotatedsecondordercone = 1;
solver(i).complex = 1;
i = i+1;

solver(i) = sdpsolver;
solver(i).tag     = 'sdpt3';
solver(i).version = '3.1';
solver(i).checkfor= {'sqlp','skron','symqmr'};
solver(i).call    = 'callsdpt331';
solver(i).constraint.equalities.linear = 1;
solver(i).constraint.inequalities.secondordercone = 1;
i = i+1;

solver(i) = sdpsolver;
solver(i).tag     = 'sdpt3';
solver(i).version = '3.02';
solver(i).checkfor= {'sqlp','skron','schursysolve'};
solver(i).call    = 'callsdpt3302';
solver(i).constraint.equalities.linear = 1;
solver(i).constraint.inequalities.secondordercone = 1;
i = i+1;

solver(i) = sdpsolver;
solver(i).tag     = 'sdpt3';
solver(i).version = '3.0';
solver(i).checkfor= {'sqlp','mexexec'};
solver(i).call    = 'callsdpt330';
solver(i).constraint.equalities.linear = 1;
solver(i).constraint.inequalities.secondordercone = 1;
i = i+1;

solver(i) = sdpsolver;
solver(i).tag     = 'sdpa';
solver(i).version = '';
solver(i).checkfor= {'sdpam.m'};
solver(i).call    = 'callsdpa';
i = i+1;

solver(i) = sdpsolver;
solver(i).tag     = 'dsdp';
solver(i).version = '5';
solver(i).checkfor= {'dsdp','dvec'};
solver(i).call    = 'calldsdp5';
i = i+1;

solver(i) = sdpsolver;
solver(i).tag     = 'dsdp';
solver(i).version = '4';
solver(i).checkfor= {'dsdp'};
solver(i).call    = 'calldsdp';
i = i+1;

solver(i) = sdpsolver;
solver(i).tag     = 'sdplr';
solver(i).version = '';
solver(i).checkfor= {'sdplr'};
solver(i).call    = 'callsdplr';
i = i+1;

solver(i) = sdpsolver;
solver(i).tag     = 'csdp';
solver(i).version = '';
solver(i).checkfor= {'csdp','readsol','writesdpa'};
solver(i).call    = 'callcsdp';
i = i+1;

solver(i) = sdpsolver;
solver(i).tag     = 'maxdet';
solver(i).version = '';
solver(i).checkfor= {'maxdet.m'};
solver(i).call    = 'callmaxdet';
solver(i).objective.maxdet = 1;
solver(i).dual = 0;
i = i+1;


% solver(i).tag     = 'specsdp';
% solver(i).version = '';
% solver(i).bmi     = 0;
% solver(i).pmi     = 0;
% solver(i).maxdet  = 0;
% solver(i).sdp     = 1;
% solver(i).kyp     = solver(i).sdp;
% solver(i).lp      = 1;
% solver(i).qp      = 0;
% solver(i).gp      = 0;
% solver(i).ncqp    = 0;
% solver(i).socp    = 0;
% solver(i).rotsocp = 0;
% solver(i).equalities = 0;
% solver(i).duals   = 1;
% solver(i).complex = 0;
% solver(i).integer = 0;
% solver(i).binary  = 0;
%solver(i).parametric = 0;
% solver(i).checkfor= {'SPSDPLSCX'};
% solver(i).call    = 'callspecsdp';
% solver(i).subcall = '';
% solver(i).show    = 1;
% i = i+1;

solver(i) = sdpsolver;
solver(i).tag     = 'pensdp';
solver(i).version = 'PENOPT';
solver(i).checkfor= {'pensdpm'};
solver(i).call    = 'callpensdpm';
i = i+1;

solver(i) = sdpsolver;
solver(i).tag     = 'pensdp';
solver(i).version = 'TOMLAB';
solver(i).checkfor= {'pensdp'};
solver(i).call    = 'callpensdp';
i = i+1;

solver(i) = sdpsolver;
solver(i).tag     = 'penbmi';
solver(i).version = 'PENOPT';
solver(i).checkfor= {'penbmim'};
solver(i).call    = 'callpenbmim';
solver(i).objective.quadratic.convex = 1;
solver(i).objective.quadratic.nonconvex = 1;
solver(i).constraint.inequalities.semidefinite.quadratic = 1;
i = i+1;

solver(i) = sdpsolver;
solver(i).tag     = 'penbmi';
solver(i).version = 'TOMLAB';
solver(i).checkfor= {'penbmi'};
solver(i).call    = 'callpenbmi';
solver(i).objective.quadratic.convex = 1;
solver(i).objective.quadratic.nonconvex = 1;
solver(i).constraint.inequalities.semidefinite.quadratic = 1;
i = i+1;

solver(i) = sdpsolver;
solver(i).tag     = 'lmilab';
solver(i).version = '';
solver(i).checkfor= {'setlmis'};
solver(i).call    = 'calllmilab';
solver(i).dual = 0;
i = i+1;

solver(i) = emptysolver;
solver(i).tag     = 'mpt';
solver(i).version = '';
solver(i).checkfor= {'mpt_mpqp'};
solver(i).call    = 'callmpt';
solver(i).objective.linear = 1;
solver(i).objective.quadratic.convex = 1;
solver(i).objective.quadratic.nonconvex = 1;
solver(i).constraint.inequalities.elementwise.linear = 1;
solver(i).parametric = 1;
i = i+1;

solver(i) = sdpsolver;
solver(i).tag     = 'kypd';
solver(i).version = '';
solver(i).checkfor= {'kypd_solver'};
solver(i).call    = 'callkypd';
i = i+1;

solver(i) = emptysolver;
solver(i).tag     = 'fmincon';
solver(i).version = '';
solver(i).checkfor= {'fmincon.m'};
solver(i).call    = 'callfmincon';
solver(i).objective.linear = 1;
solver(i).objective.quadratic.convex = 1;
solver(i).objective.quadratic.nonconvex = 1;
solver(i).objective.polynomial = 1;
solver(i).constraint.equalities.elementwise.linear = 1;
solver(i).constraint.equalities.elementwise.quadratic.convex = 1;
solver(i).constraint.equalities.elementwise.quadratic.nonconvex = 1;
solver(i).constraint.equalities.elementwise.polynomial = 1;
solver(i).constraint.inequalities.elementwise.linear = 1;
solver(i).constraint.inequalities.elementwise.quadratic.convex = 1;
solver(i).constraint.inequalities.elementwise.quadratic.nonconvex = 1;
solver(i).constraint.inequalities.elementwise.polynomial = 1;
solver(i).constraint.inequalities.semidefinite.linear = 1;
solver(i).constraint.inequalities.semidefinite.quadratic = 1;
solver(i).constraint.inequalities.semidefinite.polynomial = 1;
solver(i).dual = 1;
i = i+1;

% solver(i).tag     = 'mpcvx';
% solver(i).version = '';
% solver(i).bmi     = 0;
% solver(i).pmi     = 0;
% solver(i).maxdet  = 0;
% solver(i).sdp     = 1;
% solver(i).kyp     = solver(i).sdp;
% solver(i).lp      = 1;
% solver(i).qp      = 1;
% solver(i).gp      = 0;
% solver(i).ncqp    = 1;
% solver(i).socp    = 1;
% solver(i).rotsocp = 0;
% solver(i).qc = 0;
% solver(i).equalities = 0;
% solver(i).duals   = 0;
% solver(i).complex = 0;
% solver(i).integer = 0;
% solver(i).binary  = 0;
% solver(i).parametric = 1;
% solver(i).checkfor= {'mpcvx'};
% solver(i).call    = 'mpcvx';
% solver(i).subcall = '';
% solver(i).show    = 1;
% i = i+1;

% % ***************************************
% % SOMEWHAT MORE COMPLEX DEFINITIONS OF
% % THE INTERNAL MICP SOLVER
% % ***************************************
solver(i) = emptysolver;
solver(i).tag     = 'bnb';
solver(i).version = '';
solver(i).checkfor= {'bnb'};
solver(i).call    = 'bnb';
solver(i).objective.linear = 1;
solver(i).objective.sigmonial = 1;
solver(i).objective.quadratic.convex = 1;
solver(i).constraint.equalities.linear = 1;
solver(i).constraint.inequalities.elementwise.linear = 1;
solver(i).constraint.inequalities.elementwise.sigmonial = 1;
solver(i).constraint.inequalities.semidefinite.linear = 1;
solver(i).constraint.inequalities.secondordercone = 1;
solver(i).constraint.inequalities.rotatedsecondordercone = 1;
solver(i).constraint.integer = 1;
solver(i).constraint.binary = 1;
i = i+1;

% % ***************************************
% % SOMEWHAT MORE COMPLEX DEFINITIONS OF
% % THE INTERNAL MICP CUTTING PLANE SOLVER
% % ***************************************
% solver(i).tag     = 'cp';
% solver(i).version = '';
% solver(i).bmi     = 0;
% solver(i).pmi     = 0;
% solver(i).maxdet  = 0;
% solver(i).sdp     = 1;
% solver(i).kyp     = 0;
% solver(i).lp      = 1;
% solver(i).qp      = 0;
% solver(i).gp      = 0;
% solver(i).ncqp    = 0;
% solver(i).socp    = 0;
% solver(i).rotsocp = 0;
% solver(i).qc = 0;
% solver(i).equalities = 1;
% solver(i).duals   = 0;
% solver(i).complex = 0;
% solver(i).integer = 1;
% solver(i).binary  = 1;
% solver(i).checkfor= {'callcp'};
% solver(i).call    = 'callcp';
% solver(i).subcall = '';
% solver(i).show    = 0;
% i = i+1;

% ***************************************
% SOMEWHAT MORE COMPLEX DEFINITIONS OF
% THE INTERNAL GLOBAL BMI SOLVER
% ***************************************
solver(i) = emptysolver;
solver(i).tag     = 'bmibnb';
solver(i).version = '';
solver(i).checkfor= {'bmibnb'};
solver(i).call    = 'bmibnb';
solver(i).objective.linear = 1;
solver(i).constraint.equalities.linear = 1;
solver(i).constraint.inequalities.elementwise.linear = 1;
solver(i).constraint.inequalities.elementwise.quadratic.convex = 1;
solver(i).constraint.inequalities.elementwise.quadratic.nonconvex = 1;
solver(i).constraint.inequalities.semidefinite.linear = 1;
solver(i).constraint.inequalities.semidefinite.quadratic = 1;
solver(i).constraint.inequalities.secondordercone = 1;
solver(i).constraint.inequalities.rotatedsecondordercone = 1;
i = i+1;

solver(i) = emptysolver;
solver(i).tag     = 'none';
solver(i).version = '';
solver(i).checkfor= {'callnone.m'};
solver(i).call    = 'callnone';
solver(i).objective.linear = 1;
solver(i).constraint.equalities.linear = 1;
solver(i).constraint.inequalities.elementwise.linear = 1;
solver(i).constraint.inequalities.elementwise.quadratic.convex = 1;
solver(i).constraint.inequalities.elementwise.quadratic.nonconvex = 1;
solver(i).constraint.inequalities.semidefinite.linear = 1;
solver(i).constraint.inequalities.semidefinite.quadratic = 1;
i = i+1;





% solver(i) = emptysolver;
% solver(i).tag     = 'bmilin';
% solver(i).version = '';
% solver(i).bmi     = 1;
% solver(i).pmi     = 1;
% solver(i).maxdet  = 0;
% solver(i).sdp     = 1;
% solver(i).kyp     = 0;
% solver(i).lp      = 1;
% solver(i).qp      = 1;
% solver(i).gp      = 0;
% solver(i).ncqp    = 1;
% solver(i).socp    = 1;
% solver(i).rotsocp = 1;
% solver(i).qc = 0;
% solver(i).equalities = 1;
% solver(i).duals   = 0;
% solver(i).complex = 1;
% solver(i).integer = 0;
% solver(i).binary  = 0;
% solver(i).parametric = 0;
% solver(i).checkfor= {'callbmilin'};
% solver(i).call    = 'callbmilin';
% solver(i).subcall = '';
% solver(i).show    = 0;
% i = i+1;
% 
% solver(i) = emptysolver;
% solver(i).tag     = 'bmialt';
% solver(i).version = '';
% solver(i).bmi     = 1;
% solver(i).pmi     = 0;
% solver(i).maxdet  = 0;
% solver(i).sdp     = 1;
% solver(i).kyp     = solver(i).sdp;
% solver(i).lp      = 1;
% solver(i).qp      = 0;
% solver(i).gp      = 0;
% solver(i).ncqp    = 0;
% solver(i).socp    = 0;
% solver(i).rotsocp = 0;
% solver(i).qc = 0;
% solver(i).equalities = 0;
% solver(i).duals   = 0;
% solver(i).complex = 0;
% solver(i).integer = 0;
% solver(i).binary  = 0;
% solver(i).parametric = 0;
% solver(i).checkfor= {'callbmialt'};
% solver(i).call    = 'callbmialt';
% solver(i).subcall = '';
% solver(i).show    = 0;
% i = i+1;