function  varargout = yalmip(varargin)
%YALMIP Returns various information about YALMIP
%
%   YALMIP can be used to check version numbers and
%   find the SDPVAR and SET objects available in workspace
%
%   EXAMPLES
%    [V,D] = YALMIP('version') % Returns version and release date
%    YALMIP('nvars')           % Returns total number of declared variables
%    YALMIP('info')            % Display basic info.
%    YALMIP('solver','tag')    % Sets the solver 'solvertag' (see sdpsettings) as default solver
%
%   See also YALMIPTEST, YALMIPDEMO

% Author Johan Lfberg
% $Id: yalmip.m,v 1.24 2005/01/11 12:36:47 johanl Exp $

persistent prefered_solver
persistent internal_sdpvarstate
persistent internal_setstate

if nargin==0
    help yalmip
    return
end

if isempty(internal_sdpvarstate)
    internal_sdpvarstate.monomtable = spalloc(0,0,0);   % Polynomial powers table
    internal_sdpvarstate.variabletype = spalloc(0,0,0); % Pre-calc linear/quadratic/polynomial/sigmonial
    internal_sdpvarstate.intVariables = []; % ID of integer variables
    internal_sdpvarstate.binVariables = []; % ID of binary variables
    internal_sdpvarstate.uncVariables = []; % ID of uncertain variables (not used)
    internal_sdpvarstate.parVariables = []; % ID of parametric variables (not used)
    internal_sdpvarstate.extVariables = []; % ID of extended variables (for max,min,norm,sin, etc)
    internal_sdpvarstate.internalconstraints = [];
    internal_sdpvarstate.ExtendedMap = [];
    internal_sdpvarstate.sosid = 0;
    internal_sdpvarstate.sos_index = [];
    internal_sdpvarstate.sos_data = [];
    internal_sdpvarstate.sos_ParV = [];    
    sdpvar('setnvars',size(internal_sdpvarstate.monomtable,1));
end
if isempty(internal_setstate)
    internal_setstate.LMIid = 0;
    internal_setstate.duals_index = [];
    internal_setstate.duals_data = [];
end

spaces = ['                                    '];

switch (lower(varargin{1}))

    case 'addextendedvariable'

        % Is this operator variable already defined
        if ~isempty(internal_sdpvarstate.ExtendedMap)
            i = 1;
            while i<=length(internal_sdpvarstate.ExtendedMap)
                if isequal(varargin{2},internal_sdpvarstate.ExtendedMap(i).fcn) & isequal({varargin{3:end}}, internal_sdpvarstate.ExtendedMap(i).arg)
                    varargout{1} =  internal_sdpvarstate.ExtendedMap(i).var;
                    return
                end
                i = i + 1;
            end
        end

        varargin{2} = strrep(varargin{2},'sdpvar/',''); % Clean due to different behaviour of the function mfilename in ML 5,6 and 7
        switch varargin{2}
                            
            case {'abs'}
                % Abs is a bit special since we need one
                % new variable for each element...
                y = [];
                X = varargin{3:end};
                y = sdpvar(numel(X),1);
                for i = 1:numel(X)
                    yi = y(i);
                    internal_sdpvarstate.ExtendedMap(end+1).fcn = varargin{2};
                    internal_sdpvarstate.ExtendedMap(end).arg = {X(i)};
                    internal_sdpvarstate.ExtendedMap(end).var = yi;
                end
                internal_sdpvarstate.extVariables = [internal_sdpvarstate.extVariables getvariables(y)];
                y = reshape(y,size(X,1),size(X,2));
            case {'and','or','not'}
                y = binvar(1,1);
                internal_sdpvarstate.ExtendedMap(end+1).fcn = varargin{2};
                internal_sdpvarstate.ExtendedMap(end).arg = {varargin{3:end}};
                internal_sdpvarstate.ExtendedMap(end).var = y;
                internal_sdpvarstate.extVariables = [internal_sdpvarstate.extVariables getvariables(y)];                                
                
            otherwise                
                % This is the standard operators. INPUTS -> 1 scalar output
                y = sdpvar(1,1);
                internal_sdpvarstate.ExtendedMap(end+1).fcn = varargin{2};
                internal_sdpvarstate.ExtendedMap(end).arg = {varargin{3:end}};
                internal_sdpvarstate.ExtendedMap(end).var = y;
                internal_sdpvarstate.extVariables = [internal_sdpvarstate.extVariables getvariables(y)];                                
        end
        varargout{1} = y;
        return

    case 'extvariables'   
        varargout{1} = internal_sdpvarstate.extVariables;
    case 'extstruct'   
        found = 0;
        varargout{1} = [];
        i = 1;
        while ~found & i <=length(internal_sdpvarstate.ExtendedMap)
            if varargin{2} == getvariables(internal_sdpvarstate.ExtendedMap(i).var)
                found = 1;
                varargout{1} = internal_sdpvarstate.ExtendedMap(i);
            end
            i = i + 1;
        end        
        
    case 'lmiid'
        if not(isempty(internal_setstate.LMIid))
            internal_setstate.LMIid = internal_setstate.LMIid+1;
            varargout{1}=internal_setstate.LMIid;
        else
            internal_setstate.LMIid=1;
            varargout{1}=internal_setstate.LMIid;
        end

    case 'setnonlinearvariables'
        error('Internal error (ref. setnonlinearvariables). Report please.')

    case {'clear'}
        sdpvar('clear')
        W = evalin('caller','whos');
        for i = 1:size(W,1)
            if strcmp(W(i).class,'sdpvar') | strcmp(W(i).class,'lmi')
                evalin('caller', ['clear ' W(i).name ';']);
            end
        end

        internal_setstate.LMIid = 0;
        internal_setstate.duals_index = [];
        internal_setstate.duals_data = [];

        internal_sdpvarstate.sosid = 0;
        internal_sdpvarstate.sos_index = [];
        internal_sdpvarstate.sos_data = [];
        internal_sdpvarstate.sos_ParV = [];

        internal_sdpvarstate.monomtable = spalloc(0,0,0);
        internal_sdpvarstate.variabletype = spalloc(0,0,0);        
        internal_sdpvarstate.intVariables = [];
        internal_sdpvarstate.binVariables = [];
        internal_sdpvarstate.uncVariables = [];
        internal_sdpvarstate.parVariables = [];  
        internal_sdpvarstate.extVariables = [];  
        internal_sdpvarstate.internalconstraints = [];
        internal_sdpvarstate.ExtendedMap = [];
        sdpvar('setnvars',0);

    case 'cleardual'
        if nargin==1
            internal_setstate.duals_index = [];
            internal_setstate.duals_data = [];
        else
            if ~isempty(internal_setstate.duals_index)
                internal_setstate.lmiid = varargin{2};
                for i = 1:length(varargin{2})
                    j = find(internal_setstate.duals_index==internal_setstate.lmiid(i));
                    if ~isempty(j)
                        internal_setstate.duals_index = internal_setstate.duals_index([1:1:j-1 j+1:1:length(internal_setstate.duals_index)]);
                        internal_setstate.duals_data = {internal_setstate.duals_data{[1:1:j-1 j+1:1:length(internal_setstate.duals_data)]}};
                    end
                end
            end
        end

    case 'setdual'
        internal_setstate.duals_index = varargin{2};
        internal_setstate.duals_data = varargin{3};

    case 'dual'
        if isempty(internal_setstate.duals_index)
            varargout{1}=[];
        else
            LMIid = varargin{2};
            index_to_dual = find(LMIid==internal_setstate.duals_index);
            if isempty(index_to_dual)
                varargout{1}=[];
            else
                varargout{1} = internal_setstate.duals_data{index_to_dual};
            end
        end

    case 'clearsos'
        if nargin==1
            internal_sdpvarstate.sos_index = [];
            internal_sdpvarstate.sos_data = [];
            internal_sdpvarstate.sos_ParV = [];
        end

    case 'setsos'
        if ~isempty(internal_sdpvarstate.sos_index)
            where = find(internal_sdpvarstate.sos_index==varargin{2});
            if ~isempty(where)
                internal_sdpvarstate.sos_index(where) = varargin{2};
                internal_sdpvarstate.sos_data{where} = varargin{3};
                internal_sdpvarstate.sos_ParV{where} = varargin{4};
            else
                internal_sdpvarstate.sos_index(end+1) = varargin{2};
                internal_sdpvarstate.sos_data{end+1} = varargin{3};
                internal_sdpvarstate.sos_ParV{end+1} = varargin{4};
            end
        else
            internal_sdpvarstate.sos_index(end+1) = varargin{2};
            internal_sdpvarstate.sos_data{end+1} = varargin{3};
            internal_sdpvarstate.sos_ParV{end+1} = varargin{4};
        end

    case 'sosid'
        if not(isempty(internal_sdpvarstate.sosid))
            internal_sdpvarstate.sosid = internal_sdpvarstate.sosid+1;
            varargout{1}=internal_sdpvarstate.sosid;
        else
            internal_sdpvarstate.sosid=1;
            varargout{1}=internal_sdpvarstate.sosid;
        end

    case 'getsos'
        if isempty(internal_sdpvarstate.sos_index)
            varargout{1}=[];
            varargout{2}=[];
        else
            SOSid = varargin{2};
            index_to_sos = find(SOSid==internal_sdpvarstate.sos_index);
            if isempty(index_to_sos)
                varargout{1}=[];
                varargout{2}=[];
            else
                varargout{1} = internal_sdpvarstate.sos_data{index_to_sos};
                varargout{2} = internal_sdpvarstate.sos_ParV{index_to_sos};
                % FIX
            end
        end


    case 'getinternalsetstate'
        varargout{1} = internal_setstate; % Get internal state, called from saveobj

    case 'setinternalsetstate'
        internal_setstate = varargin{2}; % Set internal state, called from loadobj

    case 'getinternalsdpvarstate'
        varargout{1} = internal_sdpvarstate; % Get internal state, called from saveobj

    case 'setinternalsdpvarstate'
        internal_sdpvarstate = varargin{2}; % Set internal state, called from loadobj
        sdpvar('setnvars',size(internal_sdpvarstate.monomtable,1));

    case 'version'
        varargout{1}=3;
        if nargout==2
            varargout{2} = '20050111';
        end

    case 'setintvariables'
        internal_sdpvarstate.intVariables = varargin{2};

    case 'intvariables'
        varargout{1} = internal_sdpvarstate.intVariables;

    case 'setbinvariables'
        internal_sdpvarstate.binVariables = varargin{2};

    case 'binvariables'
        varargout{1} = internal_sdpvarstate.binVariables;

    case 'setuncvariables'
        internal_sdpvarstate.uncVariables = varargin{2};

    case 'uncvariables'
        varargout{1} = internal_sdpvarstate.uncVariables;

    case 'setparvariables'
        internal_sdpvarstate.parVariables = varargin{2};

    case 'parvariables'
        varargout{1} = internal_sdpvarstate.parVariables;

    case 'setmonomtable'
        internal_sdpvarstate.monomtable = varargin{2};
        sdpvar('setnvars',size(internal_sdpvarstate.monomtable,1));
        if nargin == 3
            internal_sdpvarstate.variabletype = varargin{3};
            if length(internal_sdpvarstate.variabletype) ~=size(internal_sdpvarstate.monomtable,1)
                error('ASSERT')
            end
        else
            internal_sdpvarstate.variabletype = ~(sum(internal_sdpvarstate.monomtable,2)==1 & sum(internal_sdpvarstate.monomtable~=0,2)==1);
        end

    case 'monomtable'
        varargout{1} = internal_sdpvarstate.monomtable;
        if nargout == 2            
            varargout{2} = internal_sdpvarstate.variabletype;
        end

    case 'nonlinearvariables'
        error('Internal error (ref. nonlinear variables). Report!')
        varargout{1} = internal_sdpvarstate.nonlinearvariables;
        if nargout==2
            varargout{2} = internal_sdpvarstate.nonlinearvariablesCompressed;
        end

    case {'addinternal'}
        internal_sdpvarstate.internalconstraints{end+1} = varargin{1};

    case {'setnvars'}
        sdpvar('setnvars',varargin{2});


    case {'nvars'}
        varargout{1} = sdpvar('nvars');

    case {'info'}
        [version,release] = yalmip('version');
        currentversion = num2str(version(1));
        i = 1;
        while i<length(version)
            i = i+1;
            currentversion = [currentversion '.' num2str(version(i))];
        end

        info_str = ['- - - - YALMIP ' currentversion ' ' num2str(release) ' - - - -'];

        disp(' ');
        disp(char(repmat(double('*'),1,length(info_str))));
        disp(info_str)
        disp(char(repmat(double('*'),1,length(info_str))));
        disp(' ');
        disp(['Variable     Size'])
        spaces = ['                                    '];
        ws = evalin('caller','whos');
        n = 0;
        for i = 1:size(ws,1)
            if strcmp(ws(i).class,'sdpvar')
                n = n+1;
                wsname = ws(i).name;
                wssize = [num2str(ws(i).size(1)) 'x' num2str(ws(i).size(2))];
                disp([wsname spaces(1:13-length(wsname)) wssize]);
            end
        end
        if n == 0
            disp('No SDPVAR objects found');
        end
        disp(' ');
        disp(['LMI']);
        n = 0;
        for i = 1:size(ws,1)
            if strcmp(ws(i).class,'lmi')
                n = n+1;
                wsname = ws(i).name;
                disp([wsname]);
            end
        end
        if n == 0
            disp('No SET objects found');
        end
    case 'solver'
        if (nargin==2)
            if isa(varargin{2},'char')
                solver = varargin{2};
                prefered_solver = solver;
            else
                error('Second argument should be a string with solver name');
            end
        else
            if isempty(prefered_solver)
                varargout{1}='';
            else
                varargout{1} = prefered_solver;
            end
        end
    otherwise
        disp('Command not supported');
end

